"""Tests for local fab commands."""
from django.test import TestCase

from ..fabfile.local import check, flake8, jshint


class JshintTestCase(TestCase):
    def test_command(self):
        jshint()


class Flake8TestCase(TestCase):
    def test_command(self):
        flake8()


class CheckTestCase(TestCase):
    def test_command(self):
        with self.assertRaises(SystemExit):
            check()



"""Settings that need to be set in order to run the tests."""
import os

from fabric.api import env


DEBUG = True

SITE_ID = 1

APP_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..'))


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(APP_ROOT, '../app_static')
MEDIA_ROOT = os.path.join(APP_ROOT, '../app_media')
STATICFILES_DIRS = (
    os.path.join(APP_ROOT, 'static'),
)

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'APP_DIRS': True,
    'DIRS': [os.path.join(APP_ROOT, 'tests/test_app/templates')],
    'OPTIONS': {
        'context_processors': (
            'django.contrib.auth.context_processors.auth',
            'django.template.context_processors.request',
        )
    }
}]

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
]

INTERNAL_APPS = [
    'development_fabfile',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

SECRET_KEY = 'foobar'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}

# FABFILE SETTINGS

# ============================================================================
# General settings
# ============================================================================

# This should be the folder name of your Django project
PROJECT_NAME = 'projectname'

# Path to your test_settings.py
TEST_SETTINGS_PATH = 'myproject.settings.test_settings'

# This should be the name of the virtualenv on your local machine and on your
# servers
VENV_NAME = PROJECT_NAME

DB_DUMP_FILENAME = '{0}.dump'.format(PROJECT_NAME)
MEDIA_DUMP_FILENAME = '{0}_media.tar.gz'.format(PROJECT_NAME)

# Set this to true if you want to execute makemessages during a deployment
MAKEMESSAGES_ON_DEPLOYMENT = False

# Set this to true if you want to execute compilemessages during a deployment
COMPILEMESSAGES_ON_DEPLOYMENT = False

# Add other code snippets you want to be found. Add a file type to the dict and
# define a regex, which should be processed
SYNTAX_CHECK = {
    '*.js': '(console.log|alert)',
}
# Add files or directories to exclude in the syntax check
SYNTAX_CHECK_EXCLUDES = [
    './submodules',
    'static/js/libs/',
    'node_modules',
]

JSHINT_CHECK_EXCLUDES = SYNTAX_CHECK_EXCLUDES

# Those files/dirs will be excluded in the coverage report
COVERAGE_EXCLUDES = (
    '*__init__*,*manage.py,*wsgi*,*urls*,*/settings/*,*/migrations/*,'
    '*/tests/*,*admin*,*/south_migrations/*')


# ============================================================================
# Local settings
# ============================================================================

# This should be the superuser of your postgres installation. Usually this is
# either postgres or your login username.
LOCAL_PG_ADMIN_ROLE = 'postgres'
LOCAL_PG_HOST = 'localhost'
LOCAL_COVERAGE_PATH = os.path.join(os.path.dirname(__file__), '../../coverage')

# ============================================================================
# Server settings
# ============================================================================

# This should be the name of your user that has sudo access on the server
LOGIN_USER_PROD = PROJECT_NAME
HOST_PROD = 'DOMAIN OR IP HERE'

RSYNC_EXCLUDES = [
    'local_settings.py',
    'circus.ini',
]


# These are some paths that, by convention, you set on your servers.
# You should keep them identical for all tiers (dev, stage, prod).
def get_fab_setting(setting_name):
    if setting_name == 'SERVER_REPO_ROOT':
        return '/home/{0}/src/{1}/'.format(env.user, PROJECT_NAME)

    if setting_name == 'SERVER_REPO_PROJECT_ROOT':
        # This must have trailing slash because of rsync
        return '{0}{1}/'.format(
            get_fab_setting('SERVER_REPO_ROOT'), PROJECT_NAME)

    if setting_name == 'SERVER_APP_ROOT':
        return '/home/{0}/project/'.format(env.user)

    if setting_name == 'SERVER_PROJECT_ROOT':
        return '/home/{0}/project/'.format(env.user)

    if setting_name == 'SERVER_REQUIREMENTS_PATH':
        return '{0}requirements.txt'.format(
            get_fab_setting('SERVER_REPO_PROJECT_ROOT'))

    if setting_name == 'SERVER_MEDIA_ROOT':
        return '/home/{0}/project_assets/media/'.format(env.user)

    if setting_name == 'SERVER_DB_BACKUP_DIR':
        return '/home/{0}/backups/{1}/postgres/'.format(
            env.user, PROJECT_NAME)

    if setting_name == 'SERVER_MEDIA_BACKUP_DIR':
        return '/home/{0}/backups/{1}/media/'.format(
            env.user, PROJECT_NAME)

    if setting_name == 'SERVER_WSGI_FILE':
        return '{0}{1}/wsgi.py'.format(
            get_fab_setting('SERVER_PROJECT_ROOT'), PROJECT_NAME)

    if setting_name == 'SERVER_LOCAL_ETC_DIR':
        return '/home/{0}/local/etc/'.format(env.user)


def FAB_SETTING(x):
    return get_fab_setting(x)



"""Fab tasks that execute things on a remote server."""
import sys

import django
from django.conf import settings

from distutils.version import StrictVersion
from fabric.api import cd, env, local, run

from .local import drop_db, create_db, import_db, import_media, reset_passwords
from .utils import require_server, run_workon


PYTHON_VERSION = '{}.{}'.format(sys.version_info.major, sys.version_info.minor)

if getattr(settings, 'PEM_KEY_DIR', False):
    env.key_filename = settings.PEM_KEY_DIR


@require_server
def run_collectstatic():
    """
    Runs `./manage.py collectstatic` on the given server.

    Usage::

        fab <server> run_collectstatic

    """
    with cd(settings.FAB_SETTING('SERVER_PROJECT_ROOT')):
        run_workon('python{} manage.py collectstatic --noinput'.format(
            PYTHON_VERSION))


@require_server
def run_compilemessages():
    """
    Executes ./manage.py compilemessages on the server.

    Usage::

        fab <server name> run_compilemessages

    """

    with cd(settings.FAB_SETTING('SERVER_PROJECT_ROOT')):
        run_workon('python{} manage.py compilemessages'.format(PYTHON_VERSION))


@require_server
def run_deploy_website(restart_apache=False, restart_uwsgi=False,
                       restart_nginx=False):
    """
    Executes all tasks necessary to deploy the website on the given server.

    Usage::

        fab <server> run_deploy_website

    """
    run_git_pull()
    run_pip_install()
    run_rsync_project()
    run_syncdb()
    run_collectstatic()
    if getattr(settings, 'MAKEMESSAGES_ON_DEPLOYMENT', False):
        run_makemessages()
    if getattr(settings, 'COMPILEMESSAGES_ON_DEPLOYMENT', False):
        run_compilemessages()
    if restart_apache:
        run_restart_apache()
    if restart_uwsgi:
        run_restart_uwsgi()
    if restart_nginx:
        run_restart_nginx()
    else:
        run_touch_wsgi()


@require_server
def run_download_db(filename=None):
    """
    Downloads the database from the server into your local machine.

    In order to import the downloaded database, run ``fab import_db``

    Usage::

        fab prod run_download_db
        fab prod run_download_db:filename=foobar.dump

    """
    if not filename:
        filename = settings.DB_DUMP_FILENAME
    if env.key_filename:
        ssh = settings.PROJECT_NAME
    else:
        ssh = '{0}@{1}'.format(env.user, env.host_string)
    local('scp {0}:{1}{2} .'.format(
        ssh, settings.FAB_SETTING('SERVER_DB_BACKUP_DIR'), filename))


@require_server
def run_download_media(filename=None):
    """
    Downloads the media dump from the server into your local machine.

    In order to import the downloaded media dump, run ``fab import_media``

    Usage::

        fab prod run_download_media
        fab prod run_download_media:filename=foobar.tar.gz

    """
    if not filename:
        filename = settings.MEDIA_DUMP_FILENAME
    if env.key_filename:
        ssh = settings.PROJECT_NAME
    else:
        ssh = '{0}@{1}'.format(env.user, env.host_string)
    local('scp {0}:{1}{2} .'.format(
        ssh, settings.FAB_SETTING('SERVER_MEDIA_BACKUP_DIR'), filename))


@require_server
def run_export_db(filename=None):
    """
    Exports the database on the server.

    Usage::

        fab prod run_export_db
        fab prod run_export_db:filename=foobar.dump

    """
    if not filename:
        filename = settings.DB_DUMP_FILENAME
    with cd(settings.FAB_SETTING('SERVER_PROJECT_ROOT')):
        run_workon('fab export_db:remote=True,filename={}'.format(filename))


@require_server
def run_export_media(filename=None):
    """
    Exports the media folder on the server.

    Usage::

        fab prod run_export_media
        fab prod run_export_media:filename=foobar.tar.gz

    """
    if not filename:
        filename = settings.MEDIA_DUMP_FILENAME

    with cd(settings.FAB_SETTING('SERVER_MEDIA_ROOT')):
        run('rm -rf {0}'.format(filename))
        run('tar -czf {0} *'.format(filename))
        run('mv {0} {1}'.format(
            filename, settings.FAB_SETTING('SERVER_MEDIA_BACKUP_DIR')))


@require_server
def run_git_pull():
    """
    Pulls the latest code and updates submodules.

    Usage::

        fab <server> run_git_pull

    """
    with cd(settings.FAB_SETTING('SERVER_REPO_ROOT')):
        run('git pull && git submodule init && git submodule update')


@require_server
def import_remote_db():
    """
    Downloads a db and imports it locally.

    """
    run_export_db()
    run_download_db()
    drop_db()
    create_db()
    import_db()
    reset_passwords()


@require_server
def import_remote_media():
    """
    Downloads media and imports it locally.

    """
    run_export_media()
    run_download_media()
    import_media()


@require_server
def run_makemessages():
    """
    Executes ./manage.py makemessages -s --all on the server.

    Usage::

        fab <server name> run_makemessages

    """
    with cd(settings.FAB_SETTING('SERVER_PROJECT_ROOT')):
        run_workon('python{} manage.py makemessages -s --all'.format(
            PYTHON_VERSION))


@require_server
def run_pip_install(upgrade=0):
    """
    Installs the requirement.txt file on the given server.

    Usage::

        fab <server> run_pip_install
        fab <server> run_pip_install:upgrade=1

    :param upgrade: If set to 1, the command will be executed with the
      ``--upgrade`` flag.

    """
    command = 'pip install -r {0}'.format(
        settings.FAB_SETTING('SERVER_REQUIREMENTS_PATH'))
    if upgrade:
        command += ' --upgrade'
    run_workon(command)


@require_server
def run_restart_apache():
    """
    Restarts apache on the given server.

    Usage::

        fab <server> run_restart_apache

    """
    run('{0}restart'.format(settings.FAB_SETTING('SERVER_APACHE_BIN_DIR')))


@require_server
def run_restart_uwsgi():
    """
    Restarts uwsgi on the given server.

    Usage::

        fab <server> run_restart_uwsgi

    """

    with cd(settings.FAB_SETTING('SERVER_LOCAL_ETC_DIR')):
        run('supervisorctl restart uwsgi')


@require_server
def run_restart_nginx():
    """
    Restarts uwsgi on the given server.

    Usage::

        fab <server> run_restart_nginx

    """

    with cd(settings.FAB_SETTING('SERVER_LOCAL_ETC_DIR')):
        run('supervisorctl restart nginx')


@require_server
def run_rsync_project():
    """
    Copies the project from the git repository to it's destination folder.

    This has the nice side effect of rsync deleting all ``.pyc`` files and
    removing other files that might have been left behind by sys admins messing
    around on the server.

    Usage::

        fab <server> run_rsync_project

    """
    excludes = ''
    for exclude in settings.RSYNC_EXCLUDES:
        excludes += " --exclude '{0}'".format(exclude)
    command = "rsync -avz --stats --delete {0} {1} {2}".format(
        excludes, settings.FAB_SETTING('SERVER_REPO_PROJECT_ROOT'),
        settings.FAB_SETTING('SERVER_APP_ROOT'))
    run(command)


@require_server
def run_syncdb():
    """
    Runs `./manage.py syncdb --migrate` on the given server.

    Usage::

        fab <server> run_syncdb

    """
    with cd(settings.FAB_SETTING('SERVER_PROJECT_ROOT')):
        if StrictVersion(django.get_version()) < StrictVersion('1.7'):
            run_workon('python{} manage.py syncdb --migrate --noinput'.format(
                PYTHON_VERSION))
        else:
            run_workon('python{} manage.py migrate'.format(PYTHON_VERSION))


@require_server
def run_touch_wsgi():
    """
    Runs `touch <path>/wsgi.py` on the given server.

    Usage::

        fab <server> run_touch_wsgi

    """
    run('touch {0}'.format(settings.FAB_SETTING('SERVER_WSGI_FILE')))


@require_server
def run_upload_db(filename=None):
    """
    Uploads your local database to the server.

    You can create a local dump with ``fab export_db`` first.

    In order to import the database on the server you still need to SSH into
    the server.

    Usage::

        fab prod run_upload_db
        fab prod run_upload_db:filename=foobar.dump

    """
    if not filename:
        filename = settings.DB_DUMP_FILENAME
    if env.key_filename:
        ssh = settings.PROJECT_NAME
    else:
        ssh = '{0}@{1}'.format(env.user, env.host_string)
    local('scp {0} {1}:{3}'.format(
        filename, ssh, settings.FAB_SETTING('SERVER_DB_BACKUP_DIR')))



"""
Different server options that can be used when running fab tasks.

For example, if you want to export the database from the staging server
you can call the fab task like so::

    fab stage run_export_db

"""
from django.conf import settings

from fabric.api import env


def common_conf():
    """Sets some default values in the environment."""
    env.port = '22'
    env.pg_admin_role = 'postgres'
    env.venv_name = settings.VENV_NAME
common_conf()


def local_machine():
    """Option to do something on local machine."""
    common_conf()
    env.machine = 'local'
    env.pg_admin_role = settings.LOCAL_PG_ADMIN_ROLE
    env.db_backup_dir = settings.DJANGO_PROJECT_ROOT
    env.media_backup_dir = settings.DJANGO_PROJECT_ROOT

    # Not sure what this is good for. Not used in our fabfile.
    # env.media_root = settings.DJANGO_MEDIA_ROOT
    # env.local_db_password = settings.DJANGO_DB_PASSWORD

    env.db_role = settings.DATABASES['default']['USER']
    env.db_name = settings.DATABASES['default']['NAME']


def dev():
    """Option to do something on the development server."""
    common_conf()
    env.user = settings.LOGIN_USER_DEV
    env.machine = 'dev'
    env.host_string = settings.HOST_DEV
    env.hosts = [env.host_string, ]


def stage():
    """Option to do something on the staging server."""
    common_conf()
    env.user = settings.LOGIN_USER_STAGE
    env.machine = 'stage'
    env.host_string = settings.HOST_STAGE
    env.hosts = [env.host_string, ]


def prod():
    """Option to do something on the production server."""
    common_conf()
    env.user = settings.LOGIN_USER_PROD
    env.machine = 'prod'
    env.host_string = settings.HOST_PROD
    env.hosts = [env.host_string, ]
